// Copyright (c) 2021, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 2436 $
// $LastChangedDate:: 2022-01-27 18:22:56 +0100#$
//
// This script initializes an X2C project.
// It loads the Block Scilab SCI files (in the palette manager) for diagram creation and
// loads the libaries for the Xcos simulation feature.
// It also executes user specific code located in the script 'ModelParameter.sce' if this file exists.
// The variable 'project_path' must be defined and point to the X2C project directory for this script to work properly.

// checks if project path variable is defined
if ~isdef('project_path') then
    mprintf("Project path not defined!\n");
    mprintf("Please define the variable "'project_path"' pointing to your X2C project.\n");
    return ;
else
    // checks if project path points to a valid directory in the file system
    if ~isdir(project_path) then
        mprintf("Project path does not point to a valid path in the file system!\n");
        mprintf("Invalid path: %s\n", project_path);
        return ;
    end
end

jimport("at.lcm.x2c.utils.LibraryUtils");

// get current directory
current_dir = pwd();

// get system architecture
arch = getArchitecture();

if getos() == "Linux" then
    // load X2C libraries (needed for simulation)
    loadLibs();
end

// get project directory (assumption: initProject.sce lies in X2CCode directory)
project_lib = jinvoke(jinvoke(LibraryUtils, "getLibraryRootDirectory", project_path), "toString");

mprintf("+ Initializing project\n")

// add Project specific library
chdir(project_path);
x2c_setCurrentWorkingDir();

if isdir(project_lib) then
    jimport("at.lcm.x2c.core.librarymanager.X2cLibraryManager");

    x2cLM = jinvoke(X2cLibraryManager,"getInstance");
    // use canonical path to ensure this function uses the same path than the createXcosBlock function
    // => Java function 'addPath_XcosLibPathStr' causes an error, when a Block XML is added with 2 different path strings
    // => may lead to a problem, if this function is executed directly after the createXcosBlock function is used
    project_path_canonical = jinvoke(jnewInstance("java.io.File", project_lib), "getCanonicalPath");
    x2cLibs = jinvoke(x2cLM, "addPath_XcosLibPathStr", project_path_canonical);

    for curLib = x2cLibs
        // curLib contains the full directory to the current libraries' common Scilab directory
        // => get library name by getting the parent directories' name via the Java File class (Scilab does not support such function)
        libName = jinvoke(jinvoke(jnewInstance("java.io.File", curLib), "getParentFile"), "getName");

        scilab_spec_dir = getLibraryScilabPath(fullfile(project_lib, libName));
        chdir(scilab_spec_dir);
        // load library
        loaderFile = ["loader_" + arch + ".sce"];
        if isfile(loaderFile) then
            exec(loaderFile, -1);
        else
            disp('Loader does not exist, skipped');
        end

        chdir(curLib);
        // load functions and palette
        exec("starter.sce", -1);

        mprintf("Project specific library " + libName + " added.\n");
    end
end
chdir(current_dir);

// cleanup
clear("current_dir", "arch", "project_lib", "libName", "scilab_spec_dir", "loaderFile", "project_path_canonical", "curLib");
clear("X2cLibraryManager", "LibraryUtils", "x2cLM", "x2cLM", "x2cLibs");

// execute ModelParameter script
if isfile("ModelParameter.sce") then
    exec("ModelParameter.sce", -1);
    mprintf("Model parameter loaded.\n")
end

mprintf("- Done\n");
